// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a GPL v3 license that can be
// found in the LICENSE file.

package com.heda.apk_admin;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import com.heda.apk_admin.views.ApkBackupViewer;
import com.heda.apk_admin.views.ApkControllerViewer;
import com.heda.apk_admin.views.ApkExporterViewer;
import com.heda.apk_admin.views.ApkScouterViewer;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry.Registrar;


public class ApkAdminPlugin implements FlutterPlugin, ActivityAware {
    private MethodChannel scouterChannel;
    private MethodChannel controllerChannel;
    private MethodChannel backupChannel;
    private MethodChannel exporterChannel;

    private ApkScouterViewer apkScouterViewer;
    private ApkControllerViewer apkControllerViewer;
    private ApkBackupViewer apkBackupViewer;
    private ApkExporterViewer apkExporterViewer;

    public static void registerWith(Registrar registrar) {
        ApkAdminPlugin plugin = new ApkAdminPlugin();
        plugin.setupMethodChannels(registrar.messenger(), registrar.context());
        if (registrar.activeContext() instanceof Activity) {
            plugin.startListeningToActivity(registrar.activity());
        }
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        setupMethodChannels(binding.getBinaryMessenger(), binding.getApplicationContext());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        scouterChannel = null;
        controllerChannel = null;
        backupChannel = null;
        exporterChannel = null;
        apkScouterViewer = null;
        apkControllerViewer = null;
        apkBackupViewer = null;
        apkExporterViewer = null;
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        startListeningToActivity(binding.getActivity());
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        if (apkScouterViewer != null) apkScouterViewer.activity = null;
        if (apkBackupViewer != null) apkBackupViewer.activity = null;
        if (apkExporterViewer != null) apkExporterViewer.activity = null;
    }

    private void setupMethodChannels(BinaryMessenger messenger, Context context) {
        scouterChannel = new MethodChannel(messenger, "com.heda.apk_admin.apk_scouter");
        controllerChannel = new MethodChannel(messenger, "com.heda.apk_admin.apk_controller");
        backupChannel = new MethodChannel(messenger, "com.heda.apk_admin.apk_backup");
        exporterChannel = new MethodChannel(messenger, "com.heda.apk_admin.apk_exporter");

        apkScouterViewer = new ApkScouterViewer(context);
        apkControllerViewer = new ApkControllerViewer(context);
        apkBackupViewer = new ApkBackupViewer(context);
        apkExporterViewer = new ApkExporterViewer(context);

        scouterChannel.setMethodCallHandler(apkScouterViewer);
        controllerChannel.setMethodCallHandler(apkControllerViewer);
        backupChannel.setMethodCallHandler(apkBackupViewer);
        exporterChannel.setMethodCallHandler(apkExporterViewer);
    }

    private void startListeningToActivity(Activity activity) {
        if (apkScouterViewer != null) apkScouterViewer.activity = activity;
        if (apkBackupViewer != null) apkBackupViewer.activity = activity;
        if (apkExporterViewer != null) apkExporterViewer.activity = activity;
    }
}
