package com.heda.apk_admin.core.services;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.util.Base64;

import com.heda.apk_admin.core.models.AsyncCallback;
import com.heda.apk_admin.utils.AsyncHandler;
import com.heda.apk_admin.utils.FilesController;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ApkScouter {
    private Context _context;
    private AsyncHandler asyncHandler;

    public ApkScouter(Context context) {
        _context = context;
        this.asyncHandler = new AsyncHandler();
    }

    public void getInstalledApps(
            final boolean includeSystemsApps,
            final boolean onlyIncludeAppsWithLaunchIntent,
            final boolean includeAppIcon,
            final AsyncCallback onFinished
    ) {
        asyncHandler.run(new Runnable() {
            @Override
            public void run() {
                PackageManager packageManager = _context.getPackageManager();
                List<Map<String, Object>> appsInfo = new ArrayList<>();
                Intent getInstalledApps = new Intent(Intent.ACTION_MAIN, null);
                getInstalledApps.addCategory(Intent.CATEGORY_LAUNCHER);
                List<ResolveInfo> installedApps = packageManager.queryIntentActivities(getInstalledApps, 0);
                for (ResolveInfo app : installedApps) {
                    ApplicationInfo appInfo = app.activityInfo.applicationInfo;
                    String packageName = appInfo.packageName;
                    String appName = packageManager.getApplicationLabel(appInfo).toString();
                    Intent launchIntent = packageManager.getLaunchIntentForPackage(appInfo.packageName);
                    if (!includeSystemsApps && isSystemApp(appInfo)) {
                        continue;
                    }
                    if (onlyIncludeAppsWithLaunchIntent && launchIntent == null) {
                        continue;
                    }
                    String versionName;
                    try {
                        PackageInfo packageInfo = packageManager.getPackageInfo(packageName, 0);
                        versionName = packageInfo.versionName;
                    } catch (PackageManager.NameNotFoundException e) {
                        versionName = "Unknown";
                    }
                    long installTime;
                    long lastUpdateTime;
                    try {
                        PackageInfo packageInfo = packageManager.getPackageInfo(packageName, 0);
                        installTime = packageInfo.firstInstallTime;
                        lastUpdateTime = packageInfo.lastUpdateTime;
                    } catch (PackageManager.NameNotFoundException e) {
                        installTime = 0;
                        lastUpdateTime = 0;
                    }
                    HashMap<String, Object> infoMap = new HashMap<>();
                    infoMap.put("packageName", packageName);
                    infoMap.put("appName", appName);
                    infoMap.put("versionName", versionName);
                    infoMap.put("installDate", installTime);
                    infoMap.put("lastUpdateDate", lastUpdateTime);
                    appsInfo.add(infoMap);
                    if (includeAppIcon) {
                        String encodedImage;
                        try {
                            Drawable appIcon = packageManager.getApplicationIcon(packageName);
                            encodedImage = encodeToBase64(getBitmapFromDrawable(appIcon));
                        } catch (Exception e) {
                            encodedImage = "NotFound";
                        }
                        infoMap.put("appIcon", encodedImage);
                    }
                }
                if (onFinished != null) {
                    onFinished.onAsyncCallback(appsInfo);
                }
            }
        });
        asyncHandler.stop();
    }

    public HashMap<String, String> getBackups() {
        HashMap<String, String> backups = new HashMap<>();
        FilesController filesController = new FilesController(_context);
        File backupFolder = filesController.getBackupsDir();
        File[] backupsFiles = backupFolder.listFiles();
        if (backupsFiles != null && backupsFiles.length > 0) {
            for (File backup : backupsFiles) {
                backups.put(backup.getName(), backup.getAbsolutePath());
            }
        }
        return backups;
    }

    private boolean isSystemApp(ApplicationInfo appInfo) {
        int SYSTEM_APP_MASK = ApplicationInfo.FLAG_SYSTEM | ApplicationInfo.FLAG_UPDATED_SYSTEM_APP;
        return (appInfo.flags & SYSTEM_APP_MASK) != 0;
    }

    private String encodeToBase64(Bitmap image) {
        ByteArrayOutputStream byteArrayOS = new ByteArrayOutputStream();
        image.compress(Bitmap.CompressFormat.PNG, 100, byteArrayOS);
        return Base64.encodeToString(byteArrayOS.toByteArray(), Base64.NO_WRAP);
    }

    private Bitmap getBitmapFromDrawable(Drawable drawable) {
        final Bitmap bmp = Bitmap.createBitmap(drawable.getIntrinsicWidth(), drawable.getIntrinsicHeight(), Bitmap.Config.ARGB_8888);
        final Canvas canvas = new Canvas(bmp);
        drawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
        drawable.draw(canvas);
        return bmp;
    }
}
