package com.heda.apk_admin.views;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import com.heda.apk_admin.core.models.AsyncCallback;
import com.heda.apk_admin.core.models.ChannelError;
import com.heda.apk_admin.core.services.ApkBackup;
import com.heda.apk_admin.utils.ChannelErrorHandler;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

public class ApkBackupViewer implements MethodCallHandler {
    private Context _context;
    public Activity activity;

    public ApkBackupViewer(Context context) {
        _context = context;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        ApkBackup apkBackup = new ApkBackup(_context, activity);
        try {
            switch (call.method) {
                case "makeBackup": {
                    String packageName = call.argument("packageName");
                    apkBackup.backupApp(packageName, new AsyncCallback() {
                        @Override
                        public void onAsyncCallback(final Object asyncResult) {
                            if (!activity.isFinishing()) {
                                activity.runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        result.success(asyncResult);
                                    }
                                });
                            }
                        }
                    });
                }
                break;
                case "restoreAppFromBackup": {
                    String backupPath = call.argument("apkPath");
                    boolean apkExists = apkBackup.restoreAppFromBackup(backupPath);
                    result.success(apkExists);
                }
                break;
                default:
                    result.notImplemented();
            }
        } catch (Exception e) {
            ChannelErrorHandler errorHandler = new ChannelErrorHandler();
            ChannelError channelError = errorHandler.handleChannelException(e);
            result.error(channelError.hashCode, channelError.message, channelError.cause);
        }
    }
}
