package com.heda.apk_admin.views;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import com.heda.apk_admin.core.models.AsyncCallback;
import com.heda.apk_admin.core.models.ChannelError;
import com.heda.apk_admin.core.services.ApkScouter;
import com.heda.apk_admin.utils.ChannelErrorHandler;

import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

public class ApkScouterViewer implements MethodCallHandler {
    private Context context;
    public Activity activity;

    public ApkScouterViewer(Context context) {
        this.context = context;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        ApkScouter apkScouter = new ApkScouter(context);
        try {
            switch (call.method) {
                case "getInstalledApps": {
                    boolean includeSystemApps = call.hasArgument("includeSystemApps") && (boolean) call.argument("includeSystemApps");
                    boolean onlyIncludeAppsWithLaunchIntent = call.hasArgument("onlyIncludeAppsWithLaunchIntent") && (boolean) call.argument("onlyIncludeAppsWithLaunchIntent");
                    boolean includeAppIcon = call.hasArgument("includeAppIcon") && (boolean) call.argument("includeAppIcon");
                    apkScouter.getInstalledApps(includeSystemApps, onlyIncludeAppsWithLaunchIntent, includeAppIcon, new AsyncCallback() {
                        @Override
                        public void onAsyncCallback(final Object installedApps) {
                            if (!activity.isFinishing()) {
                                activity.runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        result.success(installedApps);
                                    }
                                });
                            }
                        }
                    });
                }
                break;
                case "getBackups": {
                    Map<String, String> backups = apkScouter.getBackups();
                    result.success(backups);
                }
                break;
                default:
                    result.notImplemented();
            }
        } catch (Exception e) {
            ChannelErrorHandler errorHandler = new ChannelErrorHandler();
            ChannelError channelError = errorHandler.handleChannelException(e);
            result.error(channelError.hashCode, channelError.message, channelError.cause);
        }
    }

}
