import 'package:flutter/services.dart';

import 'package:apk_admin/utils/platform_error_handler.dart';

/// This class is specialized for the exportation logic of the plugin.
class ApkExporter {
  static const _channel = MethodChannel('com.heda.apk_admin.apk_exporter');

  /// This method redirects users to choose a receiver device from
  /// a list of all discoverable devices and then send the file.
  Future<String?> shareAppViaBluetooth(String packageName) async {
    try {
      String? shareResult = await _channel.invokeMethod(
        "shareAppViaBluetooth",
        {"packageName": packageName},
      );
      return shareResult;
    } on PlatformException catch (e) {
      throwError(e, "sharing an app");
      return "UnexpectedError";
    }
  }

  /// This method prompts a chooser to the user to choose where
  /// he wants to share the app link.
  Future<void> shareAppLink(String packageName) async {
    try {
      await _channel.invokeMethod("shareAppLink", {"packageName": packageName});
    } on PlatformException catch (e) {
      throwError(e, "sharing an app link");
    }
  }
}
