import 'dart:async';

import 'package:flutter/services.dart';

import 'package:apk_admin/models/app.dart';
import 'package:apk_admin/utils/platform_error_handler.dart';

/// This class is specialized for all the info gathering logic of the plugin.
class ApkScouter {
  static const _channel = MethodChannel('com.heda.apk_admin.apk_scouter');

  /// This method gets a list of all installed apps including appName,
  /// packageName, versionName, decodedIcon, installDate and lastUpdateDate.
  /// ```
  /// To view app icon set includeAppIcon to true and use Image.network(decodedIcon).
  Future<List<App>> getInstalledApps({
    bool includeSystemApps = false,
    bool onlyIncludeAppsWithLaunchIntent = true,
    bool includeAppIcon = false,
  }) async {
    try {
      List appsInfo = await (_channel.invokeMethod("getInstalledApps", {
        "includeSystemApps": includeSystemApps,
        "onlyIncludeAppsWithLaunchIntent": onlyIncludeAppsWithLaunchIntent,
        "includeAppIcon": includeAppIcon,
      }));
      List<App> apps = [];
      for (Map appInfo in appsInfo) {
        apps.add(App.fromMap(appInfo));
      }
      return apps;
    } on PlatformException catch (e) {
      throwError(e, "getting installed apps");
      return [];
    }
  }

  /// This method gets a map of all stored backups in the "backups" folder under your app files
  /// represented as the packageName as key and the apkPath as value.
  Future<Map?> getBackups() async {
    try {
      Map? backups = await _channel.invokeMethod("getBackups");
      return backups;
    } on PlatformException catch (e) {
      throwError(e, "sharing an app");
      return {};
    }
  }
}
